@@ -2,18 +2,8 @@ import datetime
 import hashlib
 import os
 import threading
-try:
-    import Queue
-except ImportError:
-    import queue as Queue
-try:
-    import urllib2
-except ImportError:
-    import urllib.request as urllib2
-try:
-    from urllib import parse as urlparse
-except ImportError:
-    import urlparse
+from six.moves import queue
+from six.moves import urllib
 
 from flask import Flask, jsonify, render_template, request, url_for
 from libearth.codecs import Rfc3339
@@ -28,13 +18,14 @@ from libearth.tz import now, utc
 from libearth.version import VERSION_INFO as LIBEARTH_VERSION_INFO
 from werkzeug.exceptions import HTTPException
 
+from .util import autofix_repo_url
 from .wsgi import MethodRewriteMiddleware
 
 
 app = Flask(__name__)
 app.wsgi_app = MethodRewriteMiddleware(app.wsgi_app)
 
-crawling_queue = Queue.Queue()
+crawling_queue = queue.Queue()
 
 app.config.update(
     ALLFEED='All Feeds',
@@ -50,6 +41,12 @@ except KeyError:
     pass
 
 
+@app.before_first_request
+def initialize():
+    if 'REPOSITORY' in app.config:
+        app.config['REPOSITORY'] = autofix_repo_url(app.config['REPOSITORY'])
+
+
 def crawl_category():
     running = True
     while running:
@@ -214,7 +211,7 @@ def get_stage():
             session_id = '{0}.{1}'.format(session_id, os.getpid())
 
         session = Session(session_id)
-        url = urlparse.urlparse(app.config['REPOSITORY'])
+        url = urllib.parse.urlparse(app.config['REPOSITORY'])
         if url.scheme == 'file':
             repository = FileSystemRepository(
                 url.path,
@@ -269,7 +266,7 @@ def add_feed(category_id):
     cursor = Cursor(category_id)
     url = request.form['url']
     try:
-        f = urllib2.urlopen(url)
+        f = urllib.request.urlopen(url)
         document = f.read()
         f.close()
     except Exception:
@@ -362,9 +359,7 @@ def move_outline(category_id):
         target = source.target_child
 
     dest = Cursor(category_id)
-    # TODO: use SubscriptionSet.contains() after releasing libearth 0.2
-    if isinstance(target, Category) and \
-            (category_id).startswith(source_path + '/'):
+    if isinstance(target, Category) and target.contains(dest.value):
         r = jsonify(
             error='circular-refernce',
             message='Cannot move into child element.'
@@ -437,7 +432,7 @@ def get_permalink(data):
             return max(candidates, key=lambda pair: pair[1])[0].uri
         return
     link = data.links.permalink
-    return link and link.uri
+    return link and link.uri or data.id
 
 
 def make_next_url(category_id, url_token, entry_after, read, starred,
@@ -773,20 +768,9 @@ def find_feed_and_entry(category_id, feed_id, entry_id):
             feed = stage.feeds[feed_id]
     except KeyError:
         raise FeedNotFound('The feed is not reachable')
-    feed_permalink = None
-    for link in feed.links:
-        if link.relation == 'alternate'\
-           and link.mimetype == 'text/html':
-            feed_permalink = link.uri
-        if not feed_permalink:
-            feed_permalink = feed.id
+    feed_permalink = get_permalink(feed)
     for entry in feed.entries:
-        entry_permalink = None
-        for link in entry.links:
-            if link.relation == 'alternate':
-                entry_permalink = link.uri
-        if not entry_permalink:
-            entry_permalink = entry.id
+        entry_permalink = get_permalink(entry)
         if entry_id == get_hash(entry.id):
             return feed, feed_permalink, entry, entry_permalink
     raise EntryNotFound('The entry is not reachable')
